/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarXGui::RemoteGui
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/services/tasks/RunningTask.h>

#include <ArmarXGui/interface/RemoteGuiInterface.h>
#include <ArmarXGui/libraries/RemoteGui/WidgetProxy.h>

namespace armarx::RemoteGui
{
    class RemoteGuiBase
    {
    public:
        RemoteGuiBase(const RemoteGuiInterfacePrx& remoteGui);

        void shutdown();

    protected:
        virtual void handleEvents(armarx::RemoteGui::TabProxy& tab) = 0;
        virtual const std::string& tabName() const = 0;

        armarx::RemoteGui::TabProxy& getTab();
        const armarx::RemoteGui::TabProxy& getTab() const;

        RemoteGuiInterfacePrx& getRemoteGui();
        const RemoteGuiInterfacePrx& getRemoteGui() const;

        void createTab(const WidgetPtr& widget);

    private:
        // thread function
        void run();

        RemoteGuiInterfacePrx remoteGui;
        TabProxy tab;

        RunningTask<RemoteGuiBase>::pointer_type runningTask;

    };

} // namespace armarx::RemoteGui